function [Qlty_2,Qlty_Tx]=IndSolSelKnSx(Sx1,Qsub,betaSol,M,N)
% [Qlty_2,Qlty_Tx]=IndSolSelKnSx(Sx1,Qsub,betaSol,M,N)
%
% This routine selects the individual minimal measurement solutions
% by intentional scatterers for BLUE to obtain the final solution.
%
% Input   
% Sx1:      positions of intentional scatterers
% Qsub:     measurement noise covariance matrix
% betaSol:  unknown transmitter and scatterer solutions by intentional scatterers
% M:        number of unknown transmitters
% N:        number of unknown scatterers
%
% Output   
% Qlty_2:   quality factor for solutions by intentional scatterers
% Qlty_Tx:  quality factor for transmitter solutions by intentional transmitters
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
       
%K=size(Sx1,1);
[K,Nbar]=size(Sx1);
Tx_est=reshape(betaSol(1:K*M),K,M);
Sx_est=reshape(betaSol(K*M+1:end),K,N);

for i=1:Nbar
    for j=1:M
        for k=1:N
            Sx=Sx1(:,i);
            SolTx=Tx_est(:,j);
            SolSx=Sx_est(:,k);
            alpha=atan2(SolTx(2),SolTx(1));
            beta=atan2(SolSx(2),SolSx(1));
            if K==3
                thetaSx=asin(SolSx(3)/norm(SolSx));
                thetaTx=asin(SolTx(3)/norm(SolTx));
            end
            rho_d=[((SolTx-Sx)/norm(SolTx-Sx)-SolTx/norm(SolTx))',zeros(1,K);
                ((SolTx-SolSx)/norm(SolTx-SolSx)-SolTx/norm(SolTx))',((SolSx-SolTx)/norm(SolTx-SolSx)+SolSx/norm(SolSx))'];
            rho_angleTx=[[-SolTx(2),SolTx(1),zeros(1,K-2)]./((norm([SolTx(2),SolTx(1)]))^2),zeros(1,K)];
            rho_angleSx=[zeros(1,K),[-SolSx(2),SolSx(1),zeros(1,K-2)]./((norm([SolSx(2),SolSx(1)]))^2)];
            if K==3
                rho_angleTx=[rho_angleTx;[-sin(thetaTx)*cos(alpha),-sin(thetaTx)*sin(alpha),cos(thetaTx)]/norm(SolTx),zeros(1,K)];
                rho_angleSx=[rho_angleSx;zeros(1,K),[-sin(thetaSx)*cos(beta),-sin(thetaSx)*sin(beta),cos(thetaSx)]/norm(SolSx)];
            end
            B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
            CovInd=inv(B_rhoi)*Qsub*inv(B_rhoi)';
            % FIMInd=B_rhoi'*inv(Qsub)*B_rhoi;
            Qlty_2((i-1)*M+j,k)=trace(CovInd);
            Qlty_Tx(i,j)=trace(CovInd(1:K,1:K));
        end
    end
end